// Copyright Vygintas D., 2003. All rights reserved.
//
// Permission to use, copy, modify, and distribute this software for any
// purpose is hereby granted without fee, provided that this copyright and
// permissions notice appear in all copies and derivatives.
//
// This software is provided "as is" without express or implied warranty.

#include <iterator>
#include <algorithm>
#include <iostream>

#include <boost/timer.hpp>

#include "math/tiny_vector.hpp"

template< class T, std::size_t N >
std::ostream& operator<<( std::ostream& os,
    const math::tiny_vector<T,N>& v )
{
    std::copy( v.begin(), v.end(),
        std::ostream_iterator<
            typename math::tiny_vector<T,N>::value_type >(os, " ") );
    return os; // to enable cascading
}

int main( int argc, char *argv[] )
{
    math::vector3f a(1,2,3), b(3,2,1);

    std::cout << a << std::endl;
    std::cout << b << std::endl;
    
    std::cout << std::endl;


    volatile float dist;
    
    boost::timer t;
    for( int i=0; i<1000000000; ++i )
    {
        math::tiny_vector< float, 4 > temp(1,2,3,1);
        a = temp;
        dist = a * a;
        //dist;
    }
    std::cout << dist << "; " << t.elapsed() << std::endl;
    
    dist = a.sq_length();
    std::cout << dist << std::endl;

    std::cout << a.unit() << std::endl;

    math::vector3f c(a ^ b);
    dist = c.length();
    std::cout << dist << std::endl;


    std::cout << std::endl;

    std::cout << (a == b) << std::endl;
    std::cout << (a != b) << std::endl;
    std::cout << (a < b) << std::endl;
    std::cout << (a <= b) << std::endl;
    std::cout << (a > b) << std::endl;
    std::cout << (a >= b) << std::endl;

    std::cout << std::endl;

    std::cout << (a ^ b) << std::endl;
    
    math::tiny_vector< float, 2 > temp(5,2);
    a = temp;
    
    std::cout << a << std::endl;

    std::cin.get();

    return 0;
}
